@@ -27,25 +27,30 @@
 #include "vinagre-utils.h"
 #include "vinagre-dnd.h"
 #include "vinagre-bookmarks.h"
+#include "vinagre-bookmarks-ui.h"
 #include "vinagre-window-private.h"
 #include "vinagre-bookmarks-entry.h"
-#include "vinagre-plugin.h"
 #include "vinagre-plugins-engine.h"
+#include "vinagre-commands.h"
 
 #ifdef VINAGRE_ENABLE_AVAHI
 #include "vinagre-mdns.h"
 #endif
- 
+
+#define VINAGRE_FAV_UI_XML_FILE "vinagre-fav-ui.xml"
 #define VINAGRE_FAV_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), VINAGRE_TYPE_FAV, VinagreFavPrivate))
 
 struct _VinagreFavPrivate
 {
-  VinagreWindow     *window;
-  GtkWidget         *tree;
-  GtkTreeModel      *model;
+  VinagreWindow         *window;
+  GtkWidget             *tree;
+  GtkTreeModel          *model;
+  GtkActionGroup        *action_group, *always_sensitive_action_group;
+  VinagreBookmarksEntry *selected;
+  GtkBox                *box;
 };
 
-G_DEFINE_TYPE(VinagreFav, vinagre_fav, GTK_TYPE_VBOX)
+G_DEFINE_TYPE(VinagreFav, vinagre_fav, GTK_TYPE_FRAME)
 
 /* Signals */
 enum
@@ -116,48 +121,6 @@ vinagre_fav_get_property (GObject    *object,
     }
 }
 
-static void 
-vinagre_fav_class_init (VinagreFavClass *klass)
-{
-  GObjectClass *object_class = G_OBJECT_CLASS (klass);
-
-  object_class->get_property = vinagre_fav_get_property;
-  object_class->set_property = vinagre_fav_set_property;
-
-  g_object_class_install_property (object_class,
-				   PROP_WINDOW,
-				   g_param_spec_object ("window",
-							"Window",
-							"The VinagreWindow this panel is associated with",
-							 VINAGRE_TYPE_WINDOW,
-							 G_PARAM_READWRITE |
-							 G_PARAM_CONSTRUCT_ONLY));	
-
-  signals[FAV_ACTIVATED] =
-		g_signal_new ("fav-activated",
-			      G_OBJECT_CLASS_TYPE (object_class),
-			      G_SIGNAL_RUN_FIRST,
-			      G_STRUCT_OFFSET (VinagreFavClass, fav_activated),
-			      NULL, NULL,
-			      g_cclosure_marshal_VOID__OBJECT,
-			      G_TYPE_NONE,
-			      1,
-			      G_TYPE_OBJECT);
-
-  signals[FAV_SELECTED] =
-		g_signal_new ("fav-selected",
-			      G_OBJECT_CLASS_TYPE (object_class),
-			      G_SIGNAL_RUN_FIRST,
-			      G_STRUCT_OFFSET (VinagreFavClass, fav_selected),
-			      NULL, NULL,
-			      g_cclosure_marshal_VOID__OBJECT,
-			      G_TYPE_NONE,
-			      1,
-			      G_TYPE_OBJECT);
-
-  g_type_class_add_private (object_class, sizeof (VinagreFavPrivate));
-}
-
 static void
 vinagre_fav_row_activated_cb (GtkTreeView       *treeview,
 			      GtkTreePath       *path,
@@ -185,11 +148,15 @@ vinagre_fav_row_activated_cb (GtkTreeView       *treeview,
       return;
     }
 
+  vinagre_cmd_open_bookmark (fav->priv->window,
+			     vinagre_bookmarks_entry_get_conn (entry));
+
   /* Emits the signal saying that user has activated a bookmark */
   g_signal_emit (G_OBJECT (fav), 
 		 signals[FAV_ACTIVATED],
 		 0, 
 		 entry);
+  g_object_unref (entry);
 }
 
 static void
@@ -197,7 +164,7 @@ vinagre_fav_selection_changed_cb (GtkTreeSelection *selection,
 				  VinagreFav       *fav)
 {
   GtkTreeIter iter;
-  VinagreBookmarksEntry *entry;
+  VinagreBookmarksEntry *entry = NULL;
   gboolean avahi;
 
   if (gtk_tree_selection_get_selected (selection, NULL, &iter))
@@ -207,16 +174,25 @@ vinagre_fav_selection_changed_cb (GtkTreeSelection *selection,
 			  ENTRY_COL, &entry,
                           IS_AVAHI_COL, &avahi,
 			  -1);
+      if (avahi)
+	{
+	  g_object_unref (entry);
+	  entry = NULL;
+	}
     }
 
-  if (avahi)
-    entry = NULL;
+  fav->priv->selected = entry;
+
+  gtk_action_group_set_sensitive (fav->priv->action_group,
+				  entry != NULL);
 
   /* Emits the signal saying that user has selected a bookmark */
   g_signal_emit (G_OBJECT (fav), 
 		 signals[FAV_SELECTED],
 		 0, 
 		 entry);
+  if (entry)
+    g_object_unref (entry);
 }
 
 static GtkTreePath *
@@ -243,9 +219,10 @@ menu_position (GtkMenu    *menu,
 {
   GtkTreePath *path;
   GdkRectangle rect;
-  gint wx, wy;
+  gint wy;
   GtkRequisition requisition;
   GtkWidget *w;
+  GtkAllocation allocation;
 
   w = fav->priv->tree;
   path = get_current_path (fav);
@@ -255,20 +232,20 @@ menu_position (GtkMenu    *menu,
 			       NULL,
 			       &rect);
 
-  wx = rect.x;
   wy = rect.y;
 
-  gdk_window_get_origin (w->window, x, y);
+  gdk_window_get_origin (gtk_widget_get_window (w), x, y);
 	
   gtk_widget_size_request (GTK_WIDGET (menu), &requisition);
+  gtk_widget_get_allocation (w, &allocation);
 
   if (gtk_widget_get_direction (w) == GTK_TEXT_DIR_RTL)
-    *x += w->allocation.x + w->allocation.width - requisition.width - 10;
+    *x += allocation.x + allocation.width - requisition.width - 10;
   else
-    *x += w->allocation.x + 10 ;
+    *x += allocation.x + 10 ;
 
   wy = MAX (*y + 5, *y + wy + 5);
-  wy = MIN (wy, *y + w->allocation.height - requisition.height - 5);
+  wy = MIN (wy, *y + allocation.height - requisition.height - 5);
 	
   *y = wy;
 
@@ -414,7 +391,11 @@ vinagre_fav_tooltip (GtkWidget *widget,
   gtk_tree_path_free (path);
 
   if (folder || group)
-    return FALSE;
+    {
+      if (entry)
+        g_object_unref (entry);
+      return FALSE;
+    }
 
   conn = vinagre_bookmarks_entry_get_conn (entry);
   name = vinagre_connection_get_best_name (conn);
@@ -428,6 +409,7 @@ vinagre_fav_tooltip (GtkWidget *widget,
   g_free (tip);
   g_free (name);
   g_free (uri);
+  g_object_unref (entry);
 
   return TRUE;
 }
@@ -541,6 +523,109 @@ vinagre_fav_title_cell_data_func (GtkTreeViewColumn *column,
   g_free (str);
 }
 
+static void
+vinagre_fav_bookmarks_new_folder (GtkAction  *action,
+				  VinagreFav *fav)
+{
+  vinagre_bookmarks_new_folder (vinagre_bookmarks_get_default (),
+				GTK_WINDOW (fav->priv->window));
+}
+
+static void
+vinagre_fav_bookmarks_edit (GtkAction  *action,
+			    VinagreFav *fav)
+{
+  vinagre_bookmarks_edit (vinagre_bookmarks_get_default (),
+                          fav->priv->selected,
+                          GTK_WINDOW (fav->priv->window));
+}
+
+static void
+vinagre_fav_bookmarks_del (GtkAction  *action,
+			   VinagreFav *fav)
+{
+  vinagre_bookmarks_del (vinagre_bookmarks_get_default (),
+                         fav->priv->selected,
+                         GTK_WINDOW (fav->priv->window));
+}
+
+void
+vinagre_fav_bookmarks_open (GtkAction  *action,
+			    VinagreFav *fav)
+{
+  VinagreConnection *conn;
+
+  conn = g_object_get_data (G_OBJECT (action), "conn");
+  if (!conn)
+    conn = vinagre_bookmarks_entry_get_conn (fav->priv->selected);
+
+  g_return_if_fail (VINAGRE_IS_CONNECTION (conn));
+
+  vinagre_cmd_open_bookmark (fav->priv->window, conn);
+}
+
+static const GtkActionEntry always_sensitive_actions[] =
+{
+  {"BookmarksNewFolder", "folder-new", N_("_New Folder"), NULL,
+    N_("Create a new folder"), G_CALLBACK (vinagre_fav_bookmarks_new_folder) },
+};
+
+static const GtkActionEntry actions[] =
+{
+  { "BookmarksOpen", GTK_STOCK_CONNECT, N_("_Open bookmark"), NULL,
+    N_("Connect to this machine"), G_CALLBACK (vinagre_fav_bookmarks_open) },
+  {"BookmarksEdit", GTK_STOCK_EDIT, N_("_Edit bookmark"), NULL,
+    N_("Edit the details of selected bookmark"), G_CALLBACK (vinagre_fav_bookmarks_edit) },
+  {"BookmarksDel", GTK_STOCK_DELETE, N_("_Remove from bookmarks"), NULL,
+    N_("Remove current selected connection from bookmarks"), G_CALLBACK (vinagre_fav_bookmarks_del) },
+};
+
+static void
+vinagre_fav_create_toolbar (VinagreFav *fav)
+{
+  GtkWidget *toolbar;
+  GtkActionGroup *action_group;
+  GtkUIManager *manager;
+  GError *error = NULL;
+
+  manager = fav->priv->window->priv->manager;
+  gtk_ui_manager_add_ui_from_file (manager,
+				   vinagre_fav_get_ui_xml_filename (),
+				   &error);
+  if (error != NULL)
+    {
+      g_critical (_("Could not merge UI XML file: %s"), error->message);
+      g_error_free (error);
+      return;
+    }
+
+  action_group = gtk_action_group_new ("BookmarksActionGroup");
+  gtk_action_group_set_translation_domain (action_group, NULL);
+  gtk_action_group_add_actions (action_group,
+				actions,
+				G_N_ELEMENTS (actions),
+				fav);
+  gtk_ui_manager_insert_action_group (manager, action_group, 0);
+  gtk_action_group_set_sensitive (action_group, FALSE);
+  fav->priv->action_group = action_group;
+
+  action_group = gtk_action_group_new ("BookmarksAlwaysSensitiveActionGroup");
+  gtk_action_group_set_translation_domain (action_group, NULL);
+  gtk_action_group_add_actions (action_group,
+				always_sensitive_actions,
+				G_N_ELEMENTS (always_sensitive_actions),
+				fav);
+  gtk_ui_manager_insert_action_group (manager, action_group, 0);
+  fav->priv->always_sensitive_action_group = action_group;
+
+  toolbar = gtk_ui_manager_get_widget (manager, "/FavToolBar");
+  gtk_toolbar_set_style (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_ICONS);
+  gtk_toolbar_set_icon_size (GTK_TOOLBAR (toolbar), GTK_ICON_SIZE_MENU);
+
+  gtk_widget_show (toolbar);
+  gtk_box_pack_start (fav->priv->box, toolbar, FALSE, FALSE, 0);
+}
+
 static void
 vinagre_fav_create_tree (VinagreFav *fav)
 {
@@ -558,7 +643,7 @@ vinagre_fav_create_tree (VinagreFav *fav)
   gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scroll),
 				       GTK_SHADOW_ETCHED_OUT);
 
-  gtk_box_pack_start (GTK_BOX(fav), scroll, TRUE, TRUE, 0);
+  gtk_box_pack_start (fav->priv->box, scroll, TRUE, TRUE, 0);
 
   /* Create the model */
   fav->priv->model = GTK_TREE_MODEL (gtk_tree_store_new (NUM_COLS,
@@ -642,12 +727,12 @@ drag_data_received_handl (GtkWidget *widget,
   gchar *_sdata;
   gboolean success = FALSE;
 
-  if ((selection_data != NULL) && (selection_data->length >= 0))
+  if ((selection_data != NULL) && (gtk_selection_data_get_length (selection_data) >= 0))
     {
       switch (target_type)
 	{
 	  case TARGET_VINAGRE:
-	    _sdata = (gchar*)selection_data-> data;
+	    _sdata = (gchar*)gtk_selection_data_get_data (selection_data);
 	    success = TRUE;
 	   break;
 
@@ -704,12 +789,14 @@ drag_drop_handl (GtkWidget *widget,
 {
   gboolean is_valid_drop_site;
   GdkAtom  target_type;
+  GList    *targets;
 
   is_valid_drop_site = FALSE;
+  targets = gdk_drag_context_list_targets (context);
 
-  if (context->targets)
+  if (targets)
     {
-      target_type = GDK_POINTER_TO_ATOM (g_list_nth_data (context->targets, TARGET_VINAGRE));
+      target_type = GDK_POINTER_TO_ATOM (g_list_nth_data (targets, TARGET_VINAGRE));
       gtk_drag_get_data (widget,
 			 context,
 			 target_type,
@@ -723,11 +810,84 @@ drag_drop_handl (GtkWidget *widget,
 static void
 vinagre_fav_init (VinagreFav *fav)
 {
-  GtkWidget *label_box, *label, *close_button;
-
   fav->priv = VINAGRE_FAV_GET_PRIVATE (fav);
+  fav->priv->selected = NULL;
 
-  /* setup the tree */
+  gtk_frame_set_shadow_type (GTK_FRAME (fav), GTK_SHADOW_ETCHED_IN);
+  fav->priv->box = GTK_BOX (gtk_vbox_new (FALSE, 0));
+  gtk_container_add (GTK_CONTAINER (fav), GTK_WIDGET (fav->priv->box));
+}
+
+GtkWidget *
+vinagre_fav_new (VinagreWindow *window)
+{
+  g_return_val_if_fail (VINAGRE_IS_WINDOW (window), NULL);
+
+  return GTK_WIDGET (g_object_new (VINAGRE_TYPE_FAV,
+				   "window", window,
+				   NULL));
+}
+
+static void
+vinagre_fav_hide (GtkButton *button, VinagreFav *fav)
+{
+  GtkAction *action;
+
+  action = gtk_action_group_get_action (fav->priv->window->priv->always_sensitive_action_group,
+					"ViewSidePanel");
+  gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), FALSE);
+}
+
+static void
+vinagre_fav_create_title (VinagreFav *fav)
+{
+  GtkWidget *box, *label, *close_button, *image;
+
+  box = gtk_hbox_new (FALSE, 0);
+
+ /* setup image */
+  image = gtk_image_new_from_icon_name ("user-bookmarks", GTK_ICON_SIZE_SMALL_TOOLBAR);
+  gtk_misc_set_alignment (GTK_MISC (image), 0.0, 0.5);
+  gtk_box_pack_start (GTK_BOX (box), image, FALSE, FALSE, 0);
+
+ /* setup label */
+  label = gtk_label_new (_("Bookmarks"));
+  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+  gtk_misc_set_padding (GTK_MISC (label), 6, 0);
+  gtk_box_pack_start (GTK_BOX (box), label, TRUE, TRUE, 0);
+
+  /* setup small close button */
+  close_button = vinagre_utils_create_small_close_button ();
+  gtk_box_pack_start (GTK_BOX (box), close_button, FALSE, FALSE, 0);
+  gtk_widget_set_tooltip_text (close_button, _("Hide panel"));
+  g_signal_connect (close_button,
+		    "clicked",
+		    G_CALLBACK (vinagre_fav_hide),
+		    fav);
+
+  gtk_box_pack_start (fav->priv->box, box, FALSE, FALSE, 0);
+  gtk_widget_show_all (box);
+}
+
+static void
+protocol_added_removed_cb (VinagrePluginsEngine *engine,
+			   VinagreProtocol      *protocol,
+			   VinagreFav           *fav)
+{
+  vinagre_fav_update_list (fav);
+}
+
+static void
+vinagre_fav_constructed (GObject *object)
+{
+  VinagrePluginsEngine *engine;
+  VinagreFav *fav = VINAGRE_FAV (object);
+
+  if (G_OBJECT_CLASS (vinagre_fav_parent_class)->constructed)
+    G_OBJECT_CLASS (vinagre_fav_parent_class)->constructed (object);
+
+  vinagre_fav_create_title (fav);
+  vinagre_fav_create_toolbar (fav);
   vinagre_fav_create_tree (fav);
 
   gtk_drag_dest_set (fav->priv->tree,
@@ -755,16 +915,59 @@ vinagre_fav_init (VinagreFav *fav)
                             G_CALLBACK (vinagre_fav_update_list),
                             fav);
 #endif
+
+  engine = vinagre_plugins_engine_get_default ();
+  g_signal_connect_after (engine,
+			  "protocol-added",
+			  G_CALLBACK (protocol_added_removed_cb),
+			  fav);
+  g_signal_connect_after (engine,
+			  "protocol-removed",
+			  G_CALLBACK (protocol_added_removed_cb),
+			  fav);
 }
 
-GtkWidget *
-vinagre_fav_new (VinagreWindow *window)
+static void
+vinagre_fav_class_init (VinagreFavClass *klass)
 {
-  g_return_val_if_fail (VINAGRE_IS_WINDOW (window), NULL);
+  GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-  return GTK_WIDGET (g_object_new (VINAGRE_TYPE_FAV,
-				   "window", window,
-				   NULL));
+  object_class->get_property = vinagre_fav_get_property;
+  object_class->set_property = vinagre_fav_set_property;
+  object_class->constructed  = vinagre_fav_constructed;
+
+  g_object_class_install_property (object_class,
+				   PROP_WINDOW,
+				   g_param_spec_object ("window",
+							"Window",
+							"The VinagreWindow this panel is associated with",
+							 VINAGRE_TYPE_WINDOW,
+							 G_PARAM_READWRITE |
+							 G_PARAM_CONSTRUCT_ONLY));
+
+  signals[FAV_ACTIVATED] =
+		g_signal_new ("fav-activated",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_FIRST,
+			      G_STRUCT_OFFSET (VinagreFavClass, fav_activated),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__OBJECT,
+			      G_TYPE_NONE,
+			      1,
+			      G_TYPE_OBJECT);
+
+  signals[FAV_SELECTED] =
+		g_signal_new ("fav-selected",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_FIRST,
+			      G_STRUCT_OFFSET (VinagreFavClass, fav_selected),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__OBJECT,
+			      G_TYPE_NONE,
+			      1,
+			      G_TYPE_OBJECT);
+
+  g_type_class_add_private (object_class, sizeof (VinagreFavPrivate));
 }
 
 static void
@@ -777,7 +980,7 @@ vinagre_fav_fill_bookmarks (GtkTreeStore *store, GSList *list, GtkTreeIter *pare
   GSList                *l;
   GtkTreeIter            iter;
   VinagreConnection     *conn;
-  VinagrePlugin         *plugin;
+  VinagreProtocol       *ext;
 
  for (l = list; l; l = l->next)
     {
@@ -809,11 +1012,13 @@ vinagre_fav_fill_bookmarks (GtkTreeStore *store, GSList *list, GtkTreeIter *pare
 
 	  case VINAGRE_BOOKMARKS_ENTRY_NODE_CONN:
 	    conn = vinagre_bookmarks_entry_get_conn (entry);
-	    name = vinagre_connection_get_best_name (conn);
-	    plugin = vinagre_plugins_engine_get_plugin_by_protocol (vinagre_plugins_engine_get_default (),
-								    vinagre_connection_get_protocol (conn));
+	    ext = vinagre_plugins_engine_get_plugin_by_protocol (vinagre_plugins_engine_get_default (),
+								 vinagre_connection_get_protocol (conn));
+	    if (!ext)
+	      continue;
 
-	    pixbuf = vinagre_plugin_get_icon (plugin, 16);
+	    pixbuf = vinagre_protocol_get_icon (ext, 16);
+	    name = vinagre_connection_get_best_name (conn);
 
 	    gtk_tree_store_append (store, &iter, parent_iter);
 	    gtk_tree_store_set (store, &iter,
@@ -838,10 +1043,9 @@ vinagre_fav_fill_bookmarks (GtkTreeStore *store, GSList *list, GtkTreeIter *pare
 gboolean
 vinagre_fav_update_list (VinagreFav *fav)
 {
-  GtkTreeIter        iter, parent_iter;
-  GtkTreeStore      *store;
-  GSList            *list, *l, *next;
-  GdkPixbuf         *pixbuf;
+  GtkTreeIter   parent_iter;
+  GtkTreeStore *store;
+  GSList       *list;
     
   g_return_val_if_fail (VINAGRE_IS_FAV (fav), FALSE);
 
@@ -851,14 +1055,6 @@ vinagre_fav_update_list (VinagreFav *fav)
   /* bookmarks */
   list = vinagre_bookmarks_get_all (vinagre_bookmarks_get_default ());
 
-  gtk_tree_store_append (store, &parent_iter, NULL);
-  gtk_tree_store_set (store, &parent_iter,
-                      NAME_COL, _("Bookmarks"),
-                      IS_GROUP_COL, TRUE,
-                      IS_FOLDER_COL, FALSE,
-                      IS_AVAHI_COL, FALSE,
-                      -1);
-
   vinagre_fav_fill_bookmarks (store, list, NULL, FALSE);
 
 #ifdef VINAGRE_ENABLE_AVAHI
@@ -879,4 +1075,14 @@ vinagre_fav_update_list (VinagreFav *fav)
 
   return FALSE;
 }
+
+const gchar *
+vinagre_fav_get_ui_xml_filename (void)
+{
+  if (g_file_test (VINAGRE_FAV_UI_XML_FILE, G_FILE_TEST_EXISTS))
+    return VINAGRE_FAV_UI_XML_FILE;
+  else
+    return VINAGRE_DATADIR "/" VINAGRE_FAV_UI_XML_FILE;
+}
+
 /* vim: set ts=8: */
