@@ -29,14 +29,11 @@
 #include <config.h>
 #endif
 
-/* For the workspace/viewport stuff */
-#ifdef GDK_WINDOWING_X11
-#include <gdk/gdkx.h>
-#include <X11/Xlib.h>
-#include <X11/Xutil.h>
-#include <X11/Xatom.h>
-#endif
-
+/**
+ * vinagre_utils_create_small_close_button:
+ *
+ * Return value: (transfer full):
+ */
 GtkWidget *
 vinagre_utils_create_small_close_button ()
 {
@@ -112,7 +109,7 @@ vinagre_utils_show_many_errors (const gchar *title, GSList *items, GtkWindow *pa
 void
 vinagre_utils_toggle_widget_visible (GtkWidget *widget)
 {
-  if (GTK_WIDGET_VISIBLE (widget))
+  if (gtk_widget_get_visible (widget))
     gtk_widget_hide (widget);
   else
     gtk_widget_show_all (widget);
@@ -138,29 +135,18 @@ vinagre_utils_get_ui_xml_filename (void)
 
 /**
  * vinagre_utils_get_builder:
- * @plugin: a #VinagrePlugin or NULL
- * @filename: the filename of the UI file for the plugin, or NULL if plugin is NULL
  *
- * This function gets a #GtkBuilder object for a UI file.
- * Supply a plugin and a filename in order to get a #GtkBuilder for the plugin.
- * Supply NULL to both arguments to get the main Vinagre #GtkBuilder
- *
- * Returns a #GtkBuilder on success (free with #g_object_unref) or NULL
- *  if the file cannot be found. In this case an error dialog will be shown.
+ * Return value: (allow-none) (transfer full):
  */
 GtkBuilder *
-vinagre_utils_get_builder (VinagrePlugin *plugin, const gchar *filename)
+vinagre_utils_get_builder (const gchar *filename)
 {
   GtkBuilder *xml = NULL;
   GError     *error = NULL;
-  gchar      *actual_filename, *plugin_datadir;
+  gchar      *actual_filename;
 
-  if (plugin)
-    {
-      plugin_datadir = vinagre_plugin_get_data_dir (plugin);
-      actual_filename = g_build_filename (plugin_datadir, filename, NULL);
-      g_free (plugin_datadir);
-    }
+  if (filename)
+    actual_filename = g_strdup (filename);
   else
     actual_filename = g_strdup (vinagre_utils_get_ui_filename ());
 
@@ -171,7 +157,7 @@ vinagre_utils_get_builder (VinagrePlugin *plugin, const gchar *filename)
     {
       GString *str = g_string_new (NULL);
 
-      if (plugin)
+      if (filename)
 	g_string_append (str, _("A plugin tried to open an UI file but did not succeed, with the error message:"));
       else
 	g_string_append (str, _("The program tried to open an UI file but did not succeed, with the error message:"));
@@ -255,196 +241,6 @@ vinagre_utils_handle_debug (void)
   initialized = TRUE;
 }
 
-/* the following two functions are courtesy of galeon */
-
-/**
- * gedit_utils_get_current_workspace: Get the current workspace
- *
- * Get the currently visible workspace for the #GdkScreen.
- *
- * If the X11 window property isn't found, 0 (the first workspace)
- * is returned.
- */
-guint
-vinagre_utils_get_current_workspace (GdkScreen *screen)
-{
-#ifdef GDK_WINDOWING_X11
-	GdkWindow *root_win;
-	GdkDisplay *display;
-	Atom type;
-	gint format;
-	gulong nitems;
-	gulong bytes_after;
-	guint *current_desktop;
-	gint err, result;
-	guint ret = 0;
-
-	g_return_val_if_fail (GDK_IS_SCREEN (screen), 0);
-
-	root_win = gdk_screen_get_root_window (screen);
-	display = gdk_screen_get_display (screen);
-
-	gdk_error_trap_push ();
-	result = XGetWindowProperty (GDK_DISPLAY_XDISPLAY (display), GDK_WINDOW_XID (root_win),
-				     gdk_x11_get_xatom_by_name_for_display (display, "_NET_CURRENT_DESKTOP"),
-				     0, G_MAXLONG, False, XA_CARDINAL, &type, &format, &nitems,
-				     &bytes_after, (gpointer) &current_desktop);
-	err = gdk_error_trap_pop ();
-
-	if (err != Success || result != Success)
-		return ret;
-
-	if (type == XA_CARDINAL && format == 32 && nitems > 0)
-		ret = current_desktop[0];
-
-	XFree (current_desktop);
-	return ret;
-#else
-	/* FIXME: on mac etc proably there are native APIs
-	 * to get the current workspace etc */
-	return 0;
-#endif
-}
-
-/**
- * gedit_utils_get_window_workspace: Get the workspace the window is on
- *
- * This function gets the workspace that the #GtkWindow is visible on,
- * it returns GEDIT_ALL_WORKSPACES if the window is sticky, or if
- * the window manager doesn support this function
- */
-guint
-vinagre_utils_get_window_workspace (GtkWindow *gtkwindow)
-{
-#ifdef GDK_WINDOWING_X11
-	GdkWindow *window;
-	GdkDisplay *display;
-	Atom type;
-	gint format;
-	gulong nitems;
-	gulong bytes_after;
-	guint *workspace;
-	gint err, result;
-	guint ret = VINAGRE_ALL_WORKSPACES;
-
-	g_return_val_if_fail (GTK_IS_WINDOW (gtkwindow), 0);
-	g_return_val_if_fail (GTK_WIDGET_REALIZED (GTK_WIDGET (gtkwindow)), 0);
-
-	window = GTK_WIDGET (gtkwindow)->window;
-	display = gdk_drawable_get_display (window);
-
-	gdk_error_trap_push ();
-	result = XGetWindowProperty (GDK_DISPLAY_XDISPLAY (display), GDK_WINDOW_XID (window),
-				     gdk_x11_get_xatom_by_name_for_display (display, "_NET_WM_DESKTOP"),
-				     0, G_MAXLONG, False, XA_CARDINAL, &type, &format, &nitems,
-				     &bytes_after, (gpointer) &workspace);
-	err = gdk_error_trap_pop ();
-
-	if (err != Success || result != Success)
-		return ret;
-
-	if (type == XA_CARDINAL && format == 32 && nitems > 0)
-		ret = workspace[0];
-
-	XFree (workspace);
-	return ret;
-#else
-	/* FIXME: on mac etc proably there are native APIs
-	 * to get the current workspace etc */
-	return 0;
-#endif
-}
-
-/**
- * gedit_utils_get_current_viewport: Get the current viewport origin
- *
- * Get the currently visible viewport origin for the #GdkScreen.
- *
- * If the X11 window property isn't found, (0, 0) is returned.
- */
-void
-vinagre_utils_get_current_viewport (GdkScreen    *screen,
-				  gint         *x,
-				  gint         *y)
-{
-#ifdef GDK_WINDOWING_X11
-	GdkWindow *root_win;
-	GdkDisplay *display;
-	Atom type;
-	gint format;
-	gulong nitems;
-	gulong bytes_after;
-	gulong *coordinates;
-	gint err, result;
-
-	g_return_if_fail (GDK_IS_SCREEN (screen));
-	g_return_if_fail (x != NULL && y != NULL);
-
-	/* Default values for the viewport origin */
-	*x = 0;
-	*y = 0;
-
-	root_win = gdk_screen_get_root_window (screen);
-	display = gdk_screen_get_display (screen);
-
-	gdk_error_trap_push ();
-	result = XGetWindowProperty (GDK_DISPLAY_XDISPLAY (display), GDK_WINDOW_XID (root_win),
-				     gdk_x11_get_xatom_by_name_for_display (display, "_NET_DESKTOP_VIEWPORT"),
-				     0, G_MAXLONG, False, XA_CARDINAL, &type, &format, &nitems,
-				     &bytes_after, (void*) &coordinates);
-	err = gdk_error_trap_pop ();
-
-	if (err != Success || result != Success)
-		return;
-
-	if (type != XA_CARDINAL || format != 32 || nitems < 2)
-	{
-		XFree (coordinates);
-		return;
-	}
-
-	*x = coordinates[0];
-	*y = coordinates[1];
-	XFree (coordinates);
-#else
-	/* FIXME: on mac etc proably there are native APIs
-	 * to get the current workspace etc */
-	*x = 0;
-	*y = 0;
-#endif
-}
-
-/* Make url in about dialog clickable */
-static void
-vinagre_about_dialog_handle_url (GtkAboutDialog *about,
-				 const char     *link,
-				 gpointer        data)
-{
-  GError    *error = NULL;
-  gchar     *address;
-  GdkScreen *screen;
-
-  if (g_strstr_len (link, strlen (link), "@"))
-    address = g_strdup_printf ("mailto:%s", link);
-  else
-    address = g_strdup (link);
-
-  screen = GTK_IS_WINDOW (data) ? gtk_window_get_screen (GTK_WINDOW (data)) : NULL;
-
-  gtk_show_uri (screen,
-		address,
-		GDK_CURRENT_TIME,
-		&error);
-
-  if (error != NULL) 
-    {
-      vinagre_utils_show_error (NULL, error->message, GTK_IS_WINDOW (data) ? GTK_WINDOW (data) : NULL);
-      g_error_free (error);
-    }
-
-  g_free (address);
-}
-
 void
 vinagre_utils_help_contents (GtkWindow *window, const gchar *section)
 {
@@ -509,11 +305,6 @@ vinagre_utils_help_about (GtkWindow *window)
   license_trans = g_strjoin ("\n\n", _(license[0]), _(license[1]),
 				     _(license[2]), NULL);
 
-  /* Make URLs and email clickable in about dialog */
-  gtk_about_dialog_set_url_hook (vinagre_about_dialog_handle_url, window, NULL);
-  gtk_about_dialog_set_email_hook (vinagre_about_dialog_handle_url, window, NULL);
-
-
   gtk_show_about_dialog (GTK_IS_WINDOW (window)?window:NULL,
 			 "authors", authors,
 			 "artists", artists,
@@ -539,20 +330,6 @@ vinagre_utils_parse_boolean (const gchar* value)
   return FALSE;
 }
 
-GtkWidget *
-vinagre_gtk_button_new_with_stock_icon (const gchar *label,
-                                        const gchar *stock_id)
-{
-  GtkWidget *button;
-
-  button = gtk_button_new_with_mnemonic (label);
-  gtk_button_set_image (GTK_BUTTON (button),
-            gtk_image_new_from_stock (stock_id,
-              GTK_ICON_SIZE_BUTTON));
-
-        return button;
-}
-
 /**
  * vinagre_utils_ask_question:
  * @parent: transient parent, or NULL for none
@@ -616,10 +393,10 @@ control_ok_button (GtkEditable *entry, ControlOKButton *data)
 {
   gboolean enabled = TRUE;
 
-  if (GTK_WIDGET_VISIBLE (data->uname))
+  if (gtk_widget_get_visible (data->uname))
     enabled = enabled && gtk_entry_get_text_length (GTK_ENTRY (data->uname)) > 0;
 
-  if (GTK_WIDGET_VISIBLE (data->pw))
+  if (gtk_widget_get_visible (data->pw))
     enabled = enabled && gtk_entry_get_text_length (GTK_ENTRY (data->pw)) > 0;
 
   gtk_widget_set_sensitive (data->button, enabled);
@@ -642,7 +419,7 @@ vinagre_utils_ask_credential (GtkWindow *parent,
   int             result;
   ControlOKButton control;
 
-  xml = vinagre_utils_get_builder (NULL, NULL);
+  xml = vinagre_utils_get_builder (NULL);
 
   password_dialog = GTK_WIDGET (gtk_builder_get_object (xml, "auth_required_dialog"));
   if (parent)
